#ifndef __STREAKS_TEST_H
#define __STREAKS_TEST_H

#include <QObject>

class StreaksTest : public QObject
{
    Q_OBJECT

private Q_SLOTS:
    void testSanity();
    void test00();
    void test01();
    void test02();
    void test03();
    void test04();
    void test05();
    void test06();
    void test07();
    void test08();
    void test09();
    void test10();
    void test11();
    void test12();
    void test13();
    void bench00();
};

#endif /* __STREAKS_TEST_H */
