/*
    SPDX-FileCopyrightText: 2015 Jakob Gruber <jakob.gruber@gmail.com>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "reloadableitem.h"

ReloadableItem::ReloadableItem(int x, int y) : m_x(x), m_y(y)
{
}
